@@ -12,7 +12,6 @@
 #include "processes.h"
 
 #include "kontactinterface_debug.h"
-#include <KStartupInfo>
 #include <kwindowsystem.h>
 
 #include <QDBusConnection>
@@ -20,6 +19,11 @@
 
 #include <QCommandLineParser>
 
+#include <config-kontactinterface.h>
+#if KONTACTINTERFACE_HAVE_X11
+#include <KStartupInfo>
+#endif
+
 #ifdef Q_OS_WIN
 #include <process.h>
 #endif
@@ -94,10 +98,14 @@ UniqueAppHandler::~UniqueAppHandler()
 }
 
 // DBUS call
-int UniqueAppHandler::newInstance(const QByteArray &asn_id, const QStringList &args, const QString &workingDirectory)
+int UniqueAppHandler::newInstance(const QByteArray &startupId, const QStringList &args, const QString &workingDirectory)
 {
-    if (!asn_id.isEmpty()) {
-        KStartupInfo::setStartupId(asn_id);
+    if (KWindowSystem::isPlatformX11()) {
+#if KONTACTINTERFACE_HAVE_X11
+        KStartupInfo::setStartupId(startupId);
+#endif
+    } else if (KWindowSystem::isPlatformWayland()) {
+        KWindowSystem::setCurrentXdgActivationToken(QString::fromUtf8(startupId));
     }
 
     QCommandLineParser parser;
@@ -117,10 +125,10 @@ int KontactInterface::UniqueAppHandler::activate(const QStringList &args, const
 
     if (s_mainWidget) {
         s_mainWidget->show();
-#ifdef Q_OS_WIN
-        KWindowSystem::forceActiveWindow(s_mainWidget->winId());
-#endif
+        KWindowSystem::activateWindow(s_mainWidget->windowHandle());
+#if KONTACTINTERFACE_HAVE_X11
         KStartupInfo::appStarted();
+#endif
     }
 
     // Then ensure the part appears in kontact
